//
//  AGUpdate.h
//  AraeliumUpdate
//
//  Created by Seth Willits on 8/14/12.
//  Copyright (c) 2012 Araelium Group. All rights reserved.
//

#import <Foundation/Foundation.h>



typedef void (^AGUpdateDownloadProgressHandler)(double progress);
typedef void (^AGUpdateDownloadCompletionHandler)(NSError * error);

typedef void (^AGUpdateInstallErrorHandler)(NSError * error);


enum {
	AGUpdateStatusInfoOnly = 0,
	AGUpdateStatusDownloading,
	AGUpdateStatusDownloaded,
	AGUpdateStatusUnarchiving,
	AGUpdateStatusUnarchived
};



@interface AGUpdate : NSObject
{
	NSString * title;
	NSDate * date;
	NSString * itemDescription;
	NSString * extra;
	NSURL * releaseNotesURL;
	NSString * minimumSystemVersion;
	NSString * maximumSystemVersion;
	NSURL * fileURL;
	NSString * version;
	NSString * displayVersion;
	NSDictionary * propertiesDictionary;
	
	AGUpdateDownloadProgressHandler mDownloadProgressHandler;
	AGUpdateDownloadCompletionHandler mDownloadCompletionHandler;
	NSURLDownload * mDownload;
	BOOL mIsDownloaded;
	BOOL mIsDownloading;
	NSString * mDownloadPath;
	double mDownloadProgress;
	NSUInteger mDownloadReceived;
	uint64_t mDownloadSize;
	
	AGUpdateInstallErrorHandler mInstallErrorHandler;
	BOOL mIsInstalling;
	BOOL mIsInstalled;
}

- (id)initWithDictionary:(NSDictionary *)dict failureReason:(NSString**)error;

@property (readonly, copy) NSString * title;
@property (readonly, copy) NSString * version;
@property (readonly, copy) NSString * displayVersion;
@property (readonly, copy) NSDate * date;
@property (readonly, copy) NSString * itemDescription;
@property (readonly, copy) NSString * extra;
@property (readonly, copy) NSURL * releaseNotesURL;
@property (readonly, copy) NSURL * fileURL;
@property (readonly, copy) NSString * minimumSystemVersion;
@property (readonly, copy) NSString * maximumSystemVersion;


- (void)ignore;
- (void)launch;


- (void)downloadWithProgress:(AGUpdateDownloadProgressHandler)progressHandler completion:(AGUpdateDownloadCompletionHandler)completionHandler;
- (void)cancelDownload;
@property (readonly) BOOL isDownloading;
@property (readonly) BOOL isDownloaded;
@property (readonly, copy) NSString * downloadPath;
@property (readonly) double downloadProgress;
@property (readonly) uint64_t downloadSize;


- (void)installWithErrorHandler:(AGUpdateInstallErrorHandler)errorHandler;
@property (readonly) BOOL isInstalling;
@property (readonly) BOOL isInstalled;

@end
